import os
import unittest

from django.test import TestCase
from kidotest.decorators import no_db_testcase, test_type


class NoDbTestCaseTest(TestCase):

    """
    :py:meth:`kidotest.decorators.no_db_testcase`
    """

    def test_should_raise_exception_on_cursor_db_access(self):

        @no_db_testcase
        def decorated_func(*args, **kwargs):
            from django.db import connection
            cursor = connection.cursor()
            cursor.execute('select 1;')

        self.assertRaises(RuntimeError, decorated_func)

    def test_should_raise_exception_on_orm_db_access(self):

        @no_db_testcase
        def decorated_func(*args, **kwargs):
            from django.contrib.auth.models import User
            User.objects.count()

        self.assertRaises(RuntimeError, decorated_func)


class TestTypeTest(TestCase):

    """
    :py:meth:`kidotest.decorators.test_type`
    """

    def test_should_skip_test(self):

        os.environ['IGNORE_TESTS'] = 'acceptance'

        def func():
            pass  # pragma: no coverage

        decorator = test_type('acceptance')
        decorated_func = decorator(func)

        self.assertRaises(unittest.SkipTest, decorated_func)

    def test_should_run_test(self):

        os.environ['IGNORE_TESTS'] = 'unit'

        def func():
            pass  # pragma: no coverage

        decorator = test_type('acceptance')
        decorated_func = decorator(func)

        try:
            decorated_func()
        except unittest.SkipTest:  # pragma: no coverage
            self.fail('Should not skip test')  # pragma: no coverage



from django.conf import settings
from django.test import TestCase
from mock import sentinel

from kidotest.compat import override_settings, RequestFactory


class OverrideSettingsTest(TestCase):

    """
    :py:meth:`kidotest.compat.override_settings`
    """

    def test_should_override_debug(self):

        old_debug = settings.DEBUG
        new_debug = not old_debug
        with override_settings(DEBUG=new_debug):
            self.assertEqual(settings.DEBUG, new_debug)
        self.assertEqual(settings.DEBUG, old_debug)

    @override_settings(DEBUG=sentinel.debug)
    def test_should_override_debug_with_decorator(self):

        self.assertEqual(settings.DEBUG, sentinel.debug)


@override_settings(DEBUG=sentinel.debug)
class OverrideSettingsInClassTest(TestCase):

    """
    :py:meth:`kidotest.compat.override_settings`
    """

    def test_should_override_debug_with_decorator(self):

        self.assertEqual(settings.DEBUG, sentinel.debug)


class RequestFactoryGetTest(TestCase):

    """
    :py:meth:`kidotest.compat.RequestFactory.get`
    """

    def test_should_prepare_get_request(self):

        factory = RequestFactory()
        request = factory.get('/')
        self.assertEqual(request.method, 'GET')

    def test_should_prepare_get_request_with_params(self):

        factory = RequestFactory()
        request = factory.get('/abc;type=json')
        self.assertEqual(request.method, 'GET')


class RequestFactoryPostTest(TestCase):

    """
    :py:meth:`kidotest.compat.RequestFactory.post`
    """

    def test_should_prepare_post_request(self):

        factory = RequestFactory()
        request = factory.post('/')
        self.assertEqual(request.method, 'POST')

    def test_should_prepare_post_request_with_content_type(self):

        factory = RequestFactory()
        request = factory.post('/', content_type='application/json')
        self.assertEqual(request.method, 'POST')

    def test_should_prepare_post_request_with_content_type_and_encodint(self):

        factory = RequestFactory()
        request = factory.post('/', content_type='text/plain; charset=UTF-8')
        self.assertEqual(request.method, 'POST')


class RequestFactoryHeadTest(TestCase):

    """
    :py:meth:`kidotest.compat.RequestFactory.head`
    """

    def test_should_prepare_head_request(self):

        factory = RequestFactory()
        request = factory.head('/')
        self.assertEqual(request.method, 'HEAD')


class RequestFactoryOptionsTest(TestCase):

    """
    :py:meth:`kidotest.compat.RequestFactory.options`
    """

    def test_should_prepare_options_request(self):

        factory = RequestFactory()
        request = factory.options('/')
        self.assertEqual(request.method, 'OPTIONS')


class RequestFactoryPutTest(TestCase):

    """
    :py:meth:`kidotest.compat.RequestFactory.put`
    """

    def test_should_prepare_put_request(self):

        factory = RequestFactory()
        request = factory.put('/')
        self.assertEqual(request.method, 'PUT')


class RequestFactoryDeleteTest(TestCase):

    """
    :py:meth:`kidotest.compat.RequestFactory.delete`
    """

    def test_should_prepare_delete_request(self):

        factory = RequestFactory()
        request = factory.delete('/')
        self.assertEqual(request.method, 'DELETE')



from django.conf.urls import patterns, include, url
from django.contrib import admin
from django.http import HttpResponseRedirect

urlpatterns = patterns(
    '',
    url(r'^$', lambda request: HttpResponseRedirect('/admin/')),
    url(r'^admin/', include(admin.site.urls)),
)



""" Decorators that help with running tests. """

__doc__ = """
Decorators
==========
"""

import os
import unittest

import mock


def no_db_testcase(func):
    """ Decorator that prevents your test touching database.

    :raises RuntimeError: if test try to do database query
    """
    exception = RuntimeError("Do not touch the database!")
    cursor_wrapper = mock.MagicMock(side_effect=exception)
    mocking_dict = {
        'execute': cursor_wrapper,
        'executemany': cursor_wrapper,
        'callproc': cursor_wrapper,
    }
    try:
        from django.db.backends.util import CursorWrapper
    except ImportError:
        # Django < 1.3 has no CursorWrapper so give support at least for sqlite
        from django.db.backends.sqlite3.base import SQLiteCursorWrapper as CursorWrapper
        del mocking_dict['callproc']
    try:
        CursorWrapper.execute
    except AttributeError:
        # Django < 1.6 uses __getattr__
        mocking_dict = {'__getattr__': cursor_wrapper}
    wrapped = mock.patch.multiple(CursorWrapper, **mocking_dict)
    return wrapped(func)


def test_type(name):
    """ Function creating decorators that allows skipping tests of declared type.

    :param str name: arbitrary name of test type e.g. acceptance,unit
    """
    should_ignore = (name in os.environ.get('IGNORE_TESTS', ''))
    return unittest.skipIf(should_ignore, '%s test skipped' % name)



from setuptools import setup, find_packages

with open('README.rst') as file:
    long_description = file.read()

setup(
    name="django-kidotest",
    version="1.1",
    description='Code snippets which help writing automated tests for Django.',
    long_description=long_description,
    url="http://kidosoft.pl",
    author="Jakub STOLARSKI (Dryobates)",
    author_email="jakub.stolarski@kidosoft.pl",
    license="beerware",
    keywords="django testing",
    packages=find_packages(exclude=['*tests*', '*example*']),
    install_requires=[
        'mock',
    ],
    classifiers=[
        'Development Status :: 4 - Beta',
        'Environment :: Web Environment',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: BSD License',
        'Operating System :: OS Independent',
        'Framework :: Django',
        'Programming Language :: Python',
        'Topic :: Utilities'
    ],
)



try:
    from django.test import override_settings
except ImportError:
    try:
        from django.test.utils import override_settings
    except ImportError:

        from django.conf import settings, UserSettingsHolder
        from django.utils.functional import wraps

        # Copied from django 1.3

        class override_settings(object):
            """
            Acts as either a decorator, or a context manager. If it's a decorator it
            takes a function and returns a wrapped function. If it's a contextmanager
            it's used with the ``with`` statement. In either event entering/exiting
            are called before and after, respectively, the function/block is executed.
            """
            def __init__(self, **kwargs):
                self.options = kwargs
                self.wrapped = settings._wrapped

            def __enter__(self):
                self.enable()

            def __exit__(self, exc_type, exc_value, traceback):
                self.disable()

            def __call__(self, test_func):
                from django.test import TransactionTestCase
                if isinstance(test_func, type) and issubclass(test_func, TransactionTestCase):
                    original_pre_setup = test_func._pre_setup
                    original_post_teardown = test_func._post_teardown

                    def _pre_setup(innerself):
                        self.enable()
                        original_pre_setup(innerself)

                    def _post_teardown(innerself):
                        original_post_teardown(innerself)
                        self.disable()
                    test_func._pre_setup = _pre_setup
                    test_func._post_teardown = _post_teardown
                    return test_func
                else:

                    @wraps(test_func)
                    def inner(*args, **kwargs):
                        with self:
                            return test_func(*args, **kwargs)
                return inner

            def enable(self):
                override = UserSettingsHolder(settings._wrapped)
                for key, new_value in self.options.items():
                    setattr(override, key, new_value)
                settings._wrapped = override

            def disable(self):
                settings._wrapped = self.wrapped


try:
    from django.test.client import RequestFactory
except ImportError:

    from StringIO import StringIO

    from django.http import SimpleCookie
    from django.core.handlers.wsgi import WSGIRequest
    from django.test.client import (
        encode_multipart, BOUNDARY, MULTIPART_CONTENT, CONTENT_TYPE_RE,
        smart_str, urllib, urlparse, urlencode, FakePayload)

    # Copied from django 1.3

    class RequestFactory(object):
        """
        Class that lets you create mock Request objects for use in testing.

        Usage:

        rf = RequestFactory()
        get_request = rf.get('/hello/')
        post_request = rf.post('/submit/', {'foo': 'bar'})

        Once you have a request object you can pass it to any view function,
        just as if that view had been hooked up using a URLconf.
        """
        def __init__(self, **defaults):
            self.defaults = defaults
            self.cookies = SimpleCookie()
            self.errors = StringIO()

        def _base_environ(self, **request):
            """
            The base environment for a request.
            """
            environ = {
                'HTTP_COOKIE': self.cookies.output(header='', sep='; '),
                'PATH_INFO': '/',
                'QUERY_STRING': '',
                'REMOTE_ADDR': '127.0.0.1',
                'REQUEST_METHOD': 'GET',
                'SCRIPT_NAME': '',
                'SERVER_NAME': 'testserver',
                'SERVER_PORT': '80',
                'SERVER_PROTOCOL': 'HTTP/1.1',
                'wsgi.version': (1, 0),
                'wsgi.url_scheme': 'http',
                'wsgi.errors': self.errors,
                'wsgi.multiprocess': True,
                'wsgi.multithread': False,
                'wsgi.run_once': False,
            }
            environ.update(self.defaults)
            environ.update(request)
            return environ

        def request(self, **request):
            "Construct a generic request object."
            return WSGIRequest(self._base_environ(**request))

        def _encode_data(self, data, content_type, ):
            if content_type is MULTIPART_CONTENT:
                return encode_multipart(BOUNDARY, data)
            else:
                # Encode the content so that the byte representation is correct.
                match = CONTENT_TYPE_RE.match(content_type)
                if match:
                    charset = match.group(1)
                else:
                    charset = settings.DEFAULT_CHARSET
                return smart_str(data, encoding=charset)

        def _get_path(self, parsed):
            # If there are parameters, add them
            if parsed[3]:
                return urllib.unquote(parsed[2] + ";" + parsed[3])
            else:
                return urllib.unquote(parsed[2])

        def get(self, path, data={}, **extra):
            "Construct a GET request"

            parsed = urlparse(path)
            r = {
                'CONTENT_TYPE': 'text/html; charset=utf-8',
                'PATH_INFO': self._get_path(parsed),
                'QUERY_STRING': urlencode(data, doseq=True) or parsed[4],
                'REQUEST_METHOD': 'GET',
                'wsgi.input': FakePayload('')
            }
            r.update(extra)
            return self.request(**r)

        def post(self, path, data={}, content_type=MULTIPART_CONTENT, **extra):
            "Construct a POST request."

            post_data = self._encode_data(data, content_type)

            parsed = urlparse(path)
            r = {
                'CONTENT_LENGTH': len(post_data),
                'CONTENT_TYPE': content_type,
                'PATH_INFO': self._get_path(parsed),
                'QUERY_STRING': parsed[4],
                'REQUEST_METHOD': 'POST',
                'wsgi.input': FakePayload(post_data),
            }
            r.update(extra)
            return self.request(**r)

        def head(self, path, data={}, **extra):
            "Construct a HEAD request."

            parsed = urlparse(path)
            r = {
                'CONTENT_TYPE': 'text/html; charset=utf-8',
                'PATH_INFO': self._get_path(parsed),
                'QUERY_STRING': urlencode(data, doseq=True) or parsed[4],
                'REQUEST_METHOD': 'HEAD',
                'wsgi.input': FakePayload('')
            }
            r.update(extra)
            return self.request(**r)

        def options(self, path, data={}, **extra):
            "Constrict an OPTIONS request"

            parsed = urlparse(path)
            r = {
                'PATH_INFO': self._get_path(parsed),
                'QUERY_STRING': urlencode(data, doseq=True) or parsed[4],
                'REQUEST_METHOD': 'OPTIONS',
                'wsgi.input': FakePayload('')
            }
            r.update(extra)
            return self.request(**r)

        def put(self, path, data={}, content_type=MULTIPART_CONTENT,
                **extra):
            "Construct a PUT request."

            put_data = self._encode_data(data, content_type)

            parsed = urlparse(path)
            r = {
                'CONTENT_LENGTH': len(put_data),
                'CONTENT_TYPE': content_type,
                'PATH_INFO': self._get_path(parsed),
                'QUERY_STRING': parsed[4],
                'REQUEST_METHOD': 'PUT',
                'wsgi.input': FakePayload(put_data),
            }
            r.update(extra)
            return self.request(**r)

        def delete(self, path, data={}, **extra):
            "Construct a DELETE request."

            parsed = urlparse(path)
            r = {
                'PATH_INFO': self._get_path(parsed),
                'QUERY_STRING': urlencode(data, doseq=True) or parsed[4],
                'REQUEST_METHOD': 'DELETE',
                'wsgi.input': FakePayload('')
            }
            r.update(extra)
            return self.request(**r)

__all__ = ['override_settings', 'RequestFactory']
